function calculate_mirrors,n,k,eta,d_lambda,nu,opticthick=opticthick

;+
;==============================================================================
;
;	function: calculate_mirrors
;
;	purpose : return the reflectivities and retardance of a single mirror
;
;       written : cbeck@KIS, 10/01
;
;==============================================================================
;
;       Check number of parameters.
;
;==============================================================================
if n_params() lt 5 then begin
	print
	print, "usage:	mir_par = calculate_mirrors(n,k,eta,d_lambda,nu)"
	print
        print, "        Calculate mirror reflectivities from refractive"
        print, "        indices and incidence angle."
	print
	print, "	Arguments"
        print, "                n        : refraction index"
        print, "                k        : absorption coefficient"
        print, "                d_lambda : thickness of coating in wave-length"
        print, "                           fractions d/lambda"
        print, "                nu       : refraction index of substrat"
        print, "                eta      : incidence angle on mirror [degree]"
        print
        print, "        Output"
        print, "                vector(3),[rp,rs,ret]"
        print
        print, "                rs  : reflectivity perpendicular to plain"
        print, "                          of incidence"
        print, "                rp  : reflectivity in the plain of"
        print, "                          incidence"
        print, "                ret : retardance of mirror in degrees"
        print
        print, "        Keywords:"
        print, "                opticthick : calculation as in Cap. 1989"
        print, "                             d_lambda and nu are ignored"
        print
        print, "        Calculation as in the appendix of Skumanich et al.,"
        print, "        1997"
	return,0
endif
;==============================================================================
;-

;--------------------------------------------------------------
; convert incident angle from degree into radian
;--------------------------------------------------------------
etarad = eta*!pi/180.

;--------------------------------------------------------------
; auxiliary terms 
;--------------------------------------------------------------
r1 = n^2.-k^2-(sin(etarad))^2.
r2 = 4*n^2.*k^2.
i = complex(0,1)                 ; i = sqrt(-1)

if n_elements(opticthick) ne 0 then nu = 1.

;--------------------------------------------------------------
; calculation as in Skumanich paper (1997)
;--------------------------------------------------------------
u = sqrt(1/2.*r1 +1/2.*sqrt(r1^2.+r2))
v = sqrt(-1/2.*r1+1/2.*sqrt(r1^2.+r2))
cosa = sqrt(1-(sin(etarad))^2./nu^2.)

r3 = (n+i*k)^2.
r4 = u+i*v

r_par  = (r3*cos(etarad)-r4)/(r3*cos(etarad)+r4)
r_senk = (cos(etarad)-r4)/(cos(etarad)+r4)

s_par  = (nu*r4-r3*cosa)/(nu*r4+r3*cosa)
s_senk = (r4-nu*cosa)/(r4+nu*cosa)

if n_elements(opticthick) ne 0 then begin
t = 0.
endif else t = exp(4*!pi*d_lambda*i*r4)

;print,t

R_par  = (r_par+s_par*t)/(1+r_par*s_par*t)
R_senk = (r_senk+s_senk*t)/(1+r_senk*s_senk*t)

R_x = abs(R_par)
R_y = abs(R_senk)

;--------------------------------------------------------------
; reflectivities in x and y direction
;--------------------------------------------------------------
rp = R_x^2.
rs = R_y^2.
;--------------------------------------------------------------

;--------------------------------------------------------------
; retardance
;--------------------------------------------------------------
cosd = float(R_par*conj(R_senk))/sqrt(rp*rs)
sind = -imaginary(R_par*conj(R_senk))/sqrt(rp*rs)
ret = atan(sind,cosd)                             ; as in rflctvty.pro

;--------------------------------------------------------------
; return the three mirror parameters, retardance in degrees
;--------------------------------------------------------------
return,[rp,rs,ret*180/!pi]

end
