function get_mirrors,n_coel,k_coel,n_vac,k_vac,d_lambda,nu,i1,i2,i3,opticthick = opticthick

;+
;==============================================================================
;
;	function: get_mirrors
;
;	purpose : return the reflectivities and retardance of all mirrors
;
;       calls   : calculate_mirrors.pro
;
;       edited  : cbeck@KIS, 12/01
;
;==============================================================================
;
;       Check number of parameters.
;
;==============================================================================
if n_params() lt 9 then begin
	print
	print, "usage:	mir =  get_mirrors(n_coel,k_coel,n_vac,k_vac,d_lambda"
        print, "               ,nu,i1,i2,i3)"
	print
        print, "        Calculate mirror reflectivities from refractive"
        print, "        indices and incidence angles."
	print
	print, "	Arguments"
        print, "                n_coel   : refraction index coelostat"
        print, "                k_coel   : absorption coefficient coel."
        print, "                n_vac    : refraction index main/defl."
        print, "                k_vac    : absoption coefficient main/defl."
        print, "                d_lambda : thickness of coating in wave-length"
        print, "                           fractions d/lambda"
        print, "                nu       : refraction index of substrat"
        print, "                i1,i2,i3 : incidence angle on C1, C2, main"
        print
        print, "        Output"
        print, "                vector(9),[coel1rp,coel1rs,coel1ret,coel2rp$"
        print, "                       ,coel2rs,coel2ret,prirp,prirs,priret]"
        print
        print, "                coel1rs : reflectivity perpendicular to plain"
        print, "                          of incidence, C1"
        print, "                coel1rp : reflectivity in the plain of"
        print, "                          incidence, C1"
        print, "                coel1ret: retardance of first mirror C1"
        print, "                coel2rs : reflectivity perpendicular to plain"
        print, "                          of incidence, C2"
        print, "                coel2rp : reflectivity in the plain of"
        print, "                          incidence, C2"
        print, "                coel2ret: retardance of second mirror C2"
        print, "                prirs   : reflectivity perpendicular to plain"
        print, "                          of incidence, main and deflection"
        print, "                prirp   : reflectivity in the plain of"
        print, "                          incidence, main and deflection"
        print, "                priret  : retardance of main and deflection"
        print, "                          mirror"
        print
        print, "        Keywords:"
        print, "                opticthick : calculation as in Cap. 1989"
        print, "                             d_lambda and nu are ignored"
        print
        print, "        Calculation as in the appendix of Skumanich et al.,"
        print, "        1997"
	return,0
endif
;==============================================================================
;-

;-------------------------------------------------------------------
; calculation of first coelostat mirror
;-------------------------------------------------------------------
mir_par = calculate_mirrors(n_coel,k_coel,i1,d_lambda,nu,opticthick=opticthick)
coel1rp = mir_par(0)
coel1rs = mir_par(1)
coel1ret = mir_par(2)

;-------------------------------------------------------------------
; calculation of second coelostat mirror
;-------------------------------------------------------------------
;mir_par = calculate_mirrors( n_coel,k_coel+.5,i2,d_lambda,nu,opticthick=opticthick)

mir_par = calculate_mirrors( n_coel,k_coel,i2,d_lambda,nu,opticthick=opticthick)


;mir_par = calculate_mirrors( 1.8,5.5,i2,d_lambda,nu,opticthick=opticthick)

coel2rp = mir_par(0)
coel2rs = mir_par(1)
coel2ret = mir_par(2)

;-------------------------------------------------------------------
; calculation of tel. cal. unit mounting
;-------------------------------------------------------------------
mir_par = calculate_mirrors(n_vac,k_vac,i1,d_lambda,nu,opticthick=opticthick)
calrp = mir_par(0)
calrs = mir_par(1)
calret = mir_par(2)

;-------------------------------------------------------------------
; calculation of primary & deflection mirror (same incidence angle)
;-------------------------------------------------------------------
pri_par =  calculate_mirrors(n_vac,k_vac,i3,d_lambda,nu,opticthick=opticthick)
prirp = pri_par(0)
prirs = pri_par(1)
priret = pri_par(2)

;-------------------------------------------------------------------
; return values of all four mirrors
;-------------------------------------------------------------------
return,[coel1rp,coel1rs,coel1ret,coel2rp,coel2rs,coel2ret,prirp,prirs,priret,calrp,calrs,calret]

end


