function get_tmatrix,date,mirrorparam,fitparam,azimuth,xswitch1,xi,alpha,beta,ideal = ideal,earth =earth,polarizer = polarizer,unnormalized = unnormalized,opticthick=opticthick

;+
;============================================================================
;
;	function : get_tmatrix
;
;	purpose  : return normalized telescope matrix
;
;	written :  cbeck@kis 12/01
;
;==============================================================================
;
;	Check number of arguments.
;
;==============================================================================
if (n_params() lt 6) then begin
	print
	print, "usage:  T = get_tmatrix(date,mirrorparam,fitparam,azimuth,"
        print, "                    xswitch1,xi,(alpha,beta),/ideal, /earth,"
        print, "                    /polarizer, /unnormalized)"
	print
	print, "	Use date and azimuth to construct geometry of VTT"
        print, "        coelostat and beam path. Calculate rotation and"
        print, "        incidence angles. Calculate mirror properties from"
        print, "        refraction indices. Use the fit values for the exit"
        print, "        and entry windows."
        print
	print, "	Arguments"
	print, "		date :  vector(4),[year,month,day,hour] in UT"
        print, "                mirrorparam : vector(4), [n,k,d_lambda,nu]"
        print, "                fitparam    : vector(5), [offout,winang,$"
        print, "                                      winret,exang,exret]"
        print, "                azimuth  : position of C1, positive to E"
        print, "                xswitch1 : Capitani (0) or Beck (1) angles."
        print, "                           If the \polarizer or \earth is to"
        print,"                            be used, xswitch1 must be set to 1"
        print, "                xi       : position angle of linear polarizer"
        print, "                           On C1: Positive from N to E"
        print, "                           On window: Positive from W to N"
        print, "                alpha,beta : properties of tel.cal.polarizer"
        print, "                             see vtt_ten.pro for meaning"
        print
        print, "        Keywords"
        print, "                /ideal     : use ideal mirrors"
        print, "                /earth     : fix RF0 on earth"
        print, "                polarizer =1 : put linear polarizer on C1"
        print, "                polarizer =2 : put polarizer on window"
        print, "                /opticthick : use thick coatings"
        print, "                /unnormalized : no intensity normalization"
        print
        return,0
endif
;==============================================================================
;-

;---------------------------------------------------------
; single out input
;---------------------------------------------------------
yy = date(0)
mm = date(1)
dd = date(2)
hh = date(3)

n_coel   = mirrorparam(0)
k_coel   = mirrorparam(1)
d_lambda = mirrorparam(2)
nu       = mirrorparam(3)
n_vac    = mirrorparam(4)
k_vac    =  mirrorparam(5)

offout = fitparam(0)
winang = fitparam(1)
winret = fitparam(2)
exang  = fitparam(3)
exret  = fitparam(4)

azimuth = azimuth

;---------------------------------------------------------
; get incident and rotation angles
;---------------------------------------------------------

if xswitch1 eq 0 then begin
;---------------------------------------------------------
; Capitani method
;---------------------------------------------------------

angles = get_angles(azimuth,yy,mm,dd,hh)
i1  = angles(0)
th1 = angles(1)
i2  = angles(2)
th2 = angles(3)
i3  = angles(4)
th3 = angles(5)
i4  = angles(6)
th_pol_C1 = th1
th_C1_pol = th1
th_pol_C2 = th2-th1
endif

if xswitch1 eq 1 then begin
;---------------------------------------------------------
; Beck, polarizer angles are always calculated
;---------------------------------------------------------

angles = get_angles_final(azimuth,yy,mm,dd,hh,earth=earth)
i1  = angles(0)
th1 = angles(1)
i2  = angles(2)
th2 = angles(3)
i3  = angles(4)
th3 = angles(5)
i4  = angles(6)
th_pol_C1 = angles(7)
th_C1_pol = angles(8)
th_pol_C2 = angles(9)

endif

;---------------------------------------------------------
; get single mirror parameters from incidence angles
;---------------------------------------------------------
mir = get_mirrors(n_coel,k_coel,n_vac,k_vac,d_lambda,nu,i1,i2,i3,opticthick=opticthick)

coel1rp  = mir(0)
coel1rs  = mir(1)
coel1ret = mir(2)
coel2rp  = mir(3)
coel2rs  = mir(4)
coel2ret = mir(5)
prirp    = mir(6)
prirs    = mir(7)
priret   = mir(8)
calrp    = mir(9) 
calrs    = mir(10) 
calret   = mir(11) 

;print,mir,'  Mirrorparameters'

;---------------------------------------------------------
; calculate T matrix from angles, mirrorparameters and windowproperties
;---------------------------------------------------------
T = vtt_ten(th1,th2,th3,winret,winang,exret,exang,offout,coel1rs,coel1rp,coel1ret,coel2rs,coel2rp,coel2ret,prirs,prirp,priret,calrs,calrp,calret,xi,th_pol_C1,th_C1_pol,th_pol_C2,alpha,beta,ideal = ideal,polarizer = polarizer)

;---------------------------------------------------------
; normalize T-matrix
;---------------------------------------------------------
T_out = T/T(0,0)

if n_elements(unnormalized) ne 0 then begin
T_out = T
endif

;---------------------------------------------------------
; return normalized T-matrix
;---------------------------------------------------------
return,T_out

end
