pro spinor_cal,spat_res = spat_res,no_av = no_av,plt =plt,ilimit = ilimit,retard = retard,theta_err = theta_err,theta_pol = theta_pol,gainindex = gainindex,darkindex = darkindex,cal_file = cal_file,input_dir = input_dir,output_dir = output_dir,flat_file = flat_file,swap = swap,add_cal = add_cal,nverbose = nverbose,cchi = cchi, n_separations = n_separations,nopause = nopause,maxi = maxi,tmat_save = tmat_save,wavelength = wavelength,skycal = skycal,help = help

;+
;============================================================================
;
;	procedure : spinor_cal 
;
;	purpose   : calculate SPINOR instrument response 
;
;	author    : cbeck@nso Jul 2013
;
;==============================================================================
;
;	Check number of arguments.
;
;==============================================================================
if n_elements(help) ne 0 then begin
	print
	print, "usage:   spinor_cal,spat_res = spat_res,no_av = no_av,plt =plt,ilimit = ilimit,retard = retard,theta_err = theta_err,theta_pol = theta_pol,gainindex = gainindex,darkindex = darkindex,cal_file = cal_file,input_dir = input_dir,output_dir = output_dir,flat_file = flat_file,swap = swap,add_cal = add_cal,nverbose = nverbose,cchi = cchi, n_separations = n_separations,maxi = maxi,nopause = nopause,tmat_save = tmat_save,wavelength = wavelength,help = help"
	print
	print, "	 Calculate SPINOR polarimetric instrument response."
        print, "         "
	print
	print, "	Arguments"
        print, "                none"
        print
        print, "         Keywords :"
        print, "              spat_res: use subfields in spectra to get"
        print, "                      spatially resolved X matrices"
        print, "                      spat_res=[nx,by]; nx,ny number of subfields" 
        print, "                      in x, y dimension. For tests only."
        print, "              /no_av: do not average the measurements, use"
        print, "                      previously saved polarization vectors"
        print, "              /plt : switch on plotting (recommended)"
        print, "              ilimit: required intensity limit to use pixels"
        print, "                      in the calculation, fraction of imean"
        print, "              retard: retardance value to be used in degree"
        print, "              theta_err: offset angle between retarder and"
        print, "                      polarizer, in degree"
        print, "              theta_pol: position angle of polarizer during"
        print, "                      the calibration, in degree"
        print, "              gainindex:number of gain table to be used for the"
        print, "                        reduction in the target directory"
        print, "              darkindex:number of dark current to be used for the"
        print, "                        reduction in the target directory"
        print, "              cal_file: number of calibration file to be used"
        print, "                        in the source directory."
        print, "              input_dir: source directory with raw data."
        print, "              output_dir: destination directory for reduced data."
        print, "              flat_file: use a different gain table."
        print, "                         String with the complete filepath and"
        print, "                         filename of the gain.....sav file."
        print, "               /swap   : do a swap endian of the raw data."
        print, "              add_cal: number of additional +-QU calibration"
        print, "                      files to be used in the source directory."
        print, "                      Obsolete for SPINOR. Ignore."
        print, "              /nverbose: suppress terminal output"
        print, "              cchi   : chi square value of additional +-QU"
        print, "                       vectors, measures reproduction of input"
        print, "               n_separations: fltarr(1), number of subdivisions"
        print, "                         along the slit to be used"
        print, "               maxi    : fltarr(1), consider only pixels with"
        print, "                         I < maxi * <I>"
        print, "               /nopause: ignore all stop/pause commands."
        print, "               tmat_save:  String with the complete filepath and"
        print, "                         filename of the T-matrix parameter file."
        print, "               wavelength: wavelength of data in AA. Required if"
        print, "                          T-matrix is used."
        print, "               /help   : switch to active the help text above."
        print
return
endif
;==============================================================================
;-

; set default file indices
;if n_elements(med_with) eq 0 then  med_width = 2
if n_elements(darkindex) eq 0 then  darkindex = 0
if n_elements(cal_file) eq 0 then  cal_file = 0
if n_elements(gainindex) eq 0 then gainindex = 0

; set minimum and maximum intensity limits
if n_elements(ilimit) eq 0 then ilimit = .5
if n_elements(maxi)  eq 0 then maxi = 1.5 

; number of subdivisions along the slit
if n_elements(n_separations) eq 0 then n_separations = 10.

; default ICU properties
if n_elements(retard) eq 0 then retard = 90.
if n_elements(theta_err) eq 0 then theta_err = 0.
if n_elements(theta_pol) eq 0 then theta_pol = 0.

; default wavelenght (use of telescope model)
if n_elements(wavelength) eq 0 then wavelength = 8540.

; restore averaged dark current file
ergdc = read_directory(output_dir,filter ='*dc*')
erg = check_existence(output_dir + ergdc.files(darkindex),/read,/change)
dc_file = erg(0)
restore,dc_file

; restore flat file, but it is not used anyway
if n_elements(flat_file) eq 0 then begin
   erggain = read_directory(output_dir,filter ='*gain*')
   erg = check_existence(output_dir + erggain.files(gainindex),/read,/change)
   gain_file = erg(0)
   restore,gain_file
endif else begin
   erg = check_existence(flat_file,/read,/change)
   restore,erg(0)
  
endelse

dx_final_cal = dx_final
dy_final_cal = dy_final

; get telescope matrix parameters
if n_elements(tmat_save) ne 0 then begin
   restore,tmat_save
   tparams = tt
   tparams(3) = 90.
endif

; calibration data
erg = read_directory(input_dir,filter ='*cal*')
calfiles = erg.files(where(erg.types eq 0))
calfile = calfiles(cal_file)

; output files for observed Stokes vectors and response function
erg = check_existence(output_dir+'polvec.'+calfile+'.sav',/write,/change)
outfile = erg(0)
erg = check_existence( output_dir+'xmat.'+calfile+'.sav',/write,/change)
outfile1 = erg(0)

; check input cal file
erg = check_existence(input_dir+calfile,/read)
if erg(1) ne 1 then begin
   print,'!spinor_cal: cannot read ',input_dir+calfile
   print,' Please check polarimetric calibration file.'
   print,' Aborting.'
   return
endif

read_spinor,input_dir+calfile,data,0.,hdr 
; number of steps available
nsteps = FXPAR(hdr,'NXTENS')
n = float(nsteps)

; do plotting
if n_elements(plt) ne 0 then begin
   window,0,title ='Calibration measurements'
   laodct,41
   device,decomposed = 0
endif

; subdivisions along the slit
yy = floor(dy_final/n_separations)
yyr = findgen(dy_final/yy)*yy
yyr = [yyr,dy_final]

; read in calibration data
if n_elements(no_av) eq 0 then begin

   print,' Reading in calibration  files...',calfile

   read_spinor,input_dir+calfile,data,1.,hdr 
   demod_spinor,data,i,q,u,v
   cal_img_i = i - i
   cal_img_q = i - i
   cal_img_u = i - i
   cal_img_v = i - i
   tt = size(cal_img_i)

   pos_icu = fltarr(n,5)
   pos_table = fltarr(n)
   pos_elevation =  fltarr(n)
   pos_azimuth =  fltarr(n)

   light_level = fltarr(n)
   light_level_time = fltarr(n)


; do spatially resolved response function
   if n_elements(spat_res) eq 0 then cal_curves = fltarr(n_separations,n_slits,n,4) else begin
      
      xx = floor(dx_final/float(spat_res(0)))
      yy = floor(dy_final/float(spat_res(1)))

      xxr = findgen(dx_final/xx)*xx
      yyr = findgen(dy_final/yy)*yy

      cal_curves = fltarr(n_slits,n,n_elements(xxr),n_elements(yyr),4)

      xxr = [xxr,dx_final]
      yyr = [yyr,dy_final]

   endelse

; loop over files
   for k = 0,n-1 do begin

      read_spinor,input_dir+calfile,data,1.+k,hdr 
      demod_spinor,data,i,q,u,v


      if n_elements(swap) ne 0 then begin
         i = swap_endian(i)
         q = swap_endian(q)
         u = swap_endian(u)
         v = swap_endian(v)
      endif

; update dc image with most recent dc measurement during cal
      erg_dark = strtrim(FXPAR(hdr,'PT4_FS'))
      if erg_dark eq 'DARK' then begin
         pos_icu(k,4) = 1 
         dc = i
         endif else pos_icu(k,4) = 0.

; correct for dc
      i = i - dc

      ff = where(i lt 0)
      if ff(0) ne -1 then i(ff) = 1.
      ff = where(finite(i) ne 1)
      if ff(0) ne -1 then i(ff) = 1.


; get light level
      erg_ll = strtrim(FXPAR(hdr,'DST_LLVL'))
      light_level(k) = erg_ll

; get time 
      erg_ll_time = strtrim(FXPAR(hdr,'DST_TIME'))
      time = string2num( erg_ll_time,divider = "T:")
      light_level_time(k) = time(1)+time(2)/60.+time(3)/3600.

; get ICU status (in/out, position angle)
      erg_pol = strtrim(FXPAR(hdr,'PT4_PSTG'))
      erg_ret = strtrim(FXPAR(hdr,'PT4_RSTG'))

      if erg_pol eq 'OUT' then pos_icu(k,0) = 0 else pos_icu(k,0) = 1.
      if erg_ret eq 'OUT' then pos_icu(k,2) = 0 else pos_icu(k,2) = 1.

      pos_pol = string2num(strtrim(FXPAR(hdr,'PT4_POL')))
      pos_ret = string2num(strtrim(FXPAR(hdr,'PT4_RET')))

      pos_icu(k,1) = pos_pol
      pos_icu(k,3) = pos_ret

      if pos_ret le (-1) then begin
         print
         print,'!! Warning: spinor_cal.pro !!'
         print,' Retarder position angle is: '+num2string(pos_ret)
         print,' CU retarder was stuck during calibration !'
         print
      endif


; get parameters for telescope model
      pos_azimuth(k) = FXPAR(hdr,'DST_AZ')
      pos_elevation(k) = FXPAR(hdr,'DST_EL')
      pos_table(k) = FXPAR(hdr,'DST_TBL')

 ;stop
  
; define array to store beams  
ibeams = fltarr(n_slits,2,dx+1,dy+1,4)

  
; cut out the part beams for n slits
for kk = 0,n_slits-1 do begin &  ii_temp = i & ibeams(kk,0,*,*,0) = ii_temp(posx(kk,0):posx(kk,1),posy(kk,0):posy(kk,1))  & ibeams(kk,1,*,*,0) = ii_temp(posx(kk,0):posx(kk,1),posy(kk,2):posy(kk,3)) & qq_temp = q & ibeams(kk,0,*,*,1) = qq_temp(posx(kk,0):posx(kk,1),posy(kk,0):posy(kk,1)) & ibeams(kk,1,*,*,1) = qq_temp(posx(kk,0):posx(kk,1),posy(kk,2):posy(kk,3)) & uu_temp = u & ibeams(kk,0,*,*,2) = uu_temp(posx(kk,0):posx(kk,1),posy(kk,0):posy(kk,1)) & ibeams(kk,1,*,*,2) = uu_temp(posx(kk,0):posx(kk,1),posy(kk,2):posy(kk,3)) & vv_temp = v & ibeams(kk,0,*,*,3) = vv_temp(posx(kk,0):posx(kk,1),posy(kk,0):posy(kk,1)) & ibeams(kk,1,*,*,3) = vv_temp(posx(kk,0):posx(kk,1),posy(kk,2):posy(kk,3)) & endfor


for kk =0,n_slits-1 do begin & for kkk =0,3 do begin & dd = reform(ibeams(kk,1,*,*,kkk)) & ibeams(kk,1,*,*,kkk) = reverse(dd,2) & endfor & endfor
 
     ibeams1 = fltarr(n_slits,2,dx_final+1,dy_final+1,4)

  for kk = 0,n_slits-1 do begin & ibeams1(kk,0,*,*,*) =  ibeams(kk,0,posx_final(kk,0):posx_final(kk,1), posy_final(kk,0): posy_final(kk,1),*) & ibeams1(kk,1,*,*,*) =  ibeams(kk,1,posx_final(kk,2):posx_final(kk,3), posy_final(kk,2): posy_final(kk,3),*) & endfor

 for kk = 0,n_slits -1 do begin &  for kkk = 0,3 do begin & ibeams1(kk,0,*,*,kkk) =  rot(reform(ibeams1(kk,0,*,*,kkk)),aangle(kk),/interp) & ibeams1(kk,1,*,*,kkk) =  rot(reform(ibeams1(kk,1,*,*,kkk)),aangle(kk),/interp)  & endfor & endfor

; replace outliers in intensity ; deactivated
;for kk = 0,n_slits -1 do begin & ii = reform(ibeams1(kk,0,*,*,0)) &
;ff = where(ii lt mean(ii)*.4 or ii gt mean(ii)*1.5) & ii(ff) =
;mean(ii) & ibeams1(kk,0,*,*,0) = ii & ii =
;reform(ibeams1(kk,0,*,*,1))  & ii(ff) = mean(ii) &
;ibeams1(kk,1,*,*,0) = ii & for kkk = 1,3 do begin & ii =
;reform(ibeams1(kk,0,*,*,kkk)) & ii(ff) = mean(ii) &
;ibeams1(kk,0,*,*,kkk) = ii & ii = reform(ibeams1(kk,1,*,*,kkk)) &
;ii(ff) = mean(ii) & ibeams1(kk,1,*,*,kkk) = ii  & endfor & endfor

; merge the two beams
     ibeams_merged =  fltarr(n_slits,dx_final+1,dy_final+1,4)
  for kk = 0,n_slits -1 do begin & ibeams_merged(kk,*,*,0) = ( ibeams1(kk,0,*,*,0) + ibeams1(kk,1,*,*,0) )/2. & for kkk = 1,3 do begin & ibeams_merged(kk,*,*,kkk) =    ( ibeams1(kk,0,*,*,kkk) - ibeams1(kk,1,*,*,kkk) )/2. & endfor & endfor

   if n_elements(spat_res) eq 0 then begin

; store observed values
   for kk = 0,n_slits-1 do begin  & temp_i = reform(ibeams_merged(kk,*,*,0))  & temp_q = reform(ibeams_merged(kk,*,*,1)) & temp_u = reform(ibeams_merged(kk,*,*,2)) & temp_v = reform(ibeams_merged(kk,*,*,3)) & ttemp_i = reform(ibeams1(kk,0,*,*,0)) &  ff = where(ttemp_i gt ilimit*mean(ttemp_i) and ttemp_i lt maxi*mean(ttemp_i)) & if ff(0) eq -1 then ff = where(ttemp_i eq 0) & mask = ttemp_i-ttemp_i & mask(ff) = 1. & for kkk =0,n_separations-1 do begin & i = temp_i(*,yyr(kkk):yyr(kkk+1))  & q = temp_q(*,yyr(kkk):yyr(kkk+1))  & u = temp_u(*,yyr(kkk):yyr(kkk+1)) & v = temp_v(*,yyr(kkk):yyr(kkk+1)) & mmask = mask(*,yyr(kkk):yyr(kkk+1)) & aa = where(mmask eq 1) & cal_curves(kkk,kk,k,0) = mean(i(aa)) & cal_curves(kkk,kk,k,1) = mean(q(aa)/i(aa)) & cal_curves(kkk,kk,k,2) = mean(u(aa)/i(aa)) & cal_curves(kkk,kk,k,3) = mean(v(aa)/i(aa)) & endfor & endfor

      if n_elements(verbose) ne 0 then print,'Mean IQUV: ',mean(i),mean(q),mean(u),mean(v)

   endif else begin
; get spatially resolved values
      for kk = 0,n_slits-1 do begin     
         for kkk = 0,n_elements(xxr)-2 do begin 
            for kkkk = 0,n_elements(yyr)-2 do begin 
               cal_curves(kk,k,kkk,kkkk,0) = mean(ibeams_merged(kk,xxr(kkk):xxr(kkk+1),yyr(kkkk):yyr(kkkk+1),0)) &  cal_curves(kk,k,kkk,kkkk,1) = mean(ibeams_merged(kk,xxr(kkk):xxr(kkk+1),yyr(kkkk):yyr(kkkk+1),1)/ibeams_merged(kk,xxr(kkk):xxr(kkk+1),yyr(kkkk):yyr(kkkk+1),0)) &  cal_curves(kk,k,kkk,kkkk,2) = mean(ibeams_merged(kk,xxr(kkk):xxr(kkk+1),yyr(kkkk):yyr(kkkk+1),2)/ibeams_merged(kk,xxr(kkk):xxr(kkk+1),yyr(kkkk):yyr(kkkk+1),0)) &  cal_curves(kk,k,kkk,kkkk,3) = mean(ibeams_merged(kk,xxr(kkk):xxr(kkk+1),yyr(kkkk):yyr(kkkk+1),3)/ibeams_merged(kk,xxr(kkk):xxr(kkk+1),yyr(kkkk):yyr(kkkk+1),0))
            endfor
         endfor
      endfor
   endelse

; do plotting
if n_elements(plt) ne 0 then begin

   if n_elements(spat_res) eq 0 then begin

      erase
      !p.multi = [4,2,2]
      plot,cal_curves(0,0,*,0),/nodata,yr = [0,max(cal_curves(0,0,*,0))],/ystyle

   for kkk = 0,n_separations-1 do begin & for kk = 0,n_slits-1 do begin  oplot,cal_curves(kkk,kk,*,0),color =40*(kk+kkk)+10 & endfor & endfor
  
      plot,cal_curves(0,0,*,1),/nodata,yr = [-1,1],/ystyle
     for kkk = 0,n_separations-1 do begin & for kk = 0,n_slits-1 do begin  oplot,cal_curves(kkk,kk,*,1),color =40*(kk+kkk)+10 & endfor & endfor

        plot,cal_curves(0,0,*,2),/nodata,yr = [-1,1],/ystyle
   for kkk = 0,n_separations-1 do begin & for kk = 0,n_slits-1 do begin  oplot,cal_curves(kkk,kk,*,2),color =40*(kk+kkk)+10 & endfor & endfor

      plot,cal_curves(0,0,*,3),/nodata,yr = [-1,1],/ystyle
     for kkk = 0,n_separations-1 do begin & for kk = 0,n_slits-1 do begin  oplot,cal_curves(kkk,kk,*,3),color =40*(kk+kkk)+10 & endfor & endfor
  
      wait,.1

   endif else begin

      erase
      !p.multi = [4,2,2]
      plot,cal_curves(0,*,0,0,0)
   for kk = 1,n_slits-1 do begin & for kkk = 0,n_elements(xxr)-2 do begin & for kkkk = 0,n_elements(yyr)-2 do begin oplot,cal_curves(kk,*,kkk,kkkk,0),color =10*(kk+kkk+kkkk) & endfor & endfor & endfor

      plot,cal_curves(0,*,0,0,1)
   for kk = 1,n_slits-1 do begin & for kkk = 0,n_elements(xxr)-2 do begin & for kkkk = 0,n_elements(yyr)-2 do begin oplot,cal_curves(kk,*,kkk,kkkk,1),color =10*(kk+kkk+kkkk) & endfor & endfor & endfor
      plot,cal_curves(0,*,0,0,2)
   for kk = 1,n_slits-1 do begin & for kkk = 0,n_elements(xxr)-2 do begin & for kkkk = 0,n_elements(yyr)-2 do begin oplot,cal_curves(kk,*,kkk,kkkk,2),color =10*(kk+kkk+kkkk) & endfor & endfor & endfor

      plot,cal_curves(0,*,0,0,3)
   for kk = 1,n_slits-1 do begin & for kkk = 0,n_elements(xxr)-2 do begin & for kkkk = 0,n_elements(yyr)-2 do begin oplot,cal_curves(kk,*,kkk,kkkk,3),color =10*(kk+kkk+kkkk) & endfor & endfor & endfor

   endelse
endif

endfor

;pos_icu(*,3) = [0,0,0,45,90,135,0,0,0,0,315,337.5,0,22.5,45,0,22.5,45,67.5,90,45,67.5,90,112.5,135,90,112.5,135,157,180,0]


;0,0,0,45,90,135,0,0,0,0,315,337.5,0,22.5,45,0,22.5,45,67.5,90,45,67.5,90,112.5,135,90,112.5,135,157,180,0,0,0,0,45,90,135,0,0,0,0,315,337.5,0,22.5,45,0,22.5,45,67.5,90,45,67.5,90,112.5,135,90,112.5,135,157,180,0,0,0,0,45,90,135,0,0,0,0,315,337.5,0,22.5,45,0,22.5,45,67.5,90,45,67.5,90,112.5,135,90,112.5,135,157,180,0]


pp = 0
if pp eq 1 then begin
window,0


cchars =.7
set_plot,'PS'
device,filename = '/export/cbeck/work/DKIST/telcal/SPINOR_sequence.ps',xs = 17,ys = 14,/color,bits_per_pixel = 8

      !p.multi = [4,2,2]
      plot,cal_curves(0,0,*,0),/nodata,yr = [0,max(cal_curves(0,0,*,0))],/ystyle,title = 'Stokes I',charsize = cchars, ytitle = 'counts',xtitle = 'calibration position'

   for kkk = 0,n_separations-1 do begin & for kk = 0,n_slits-1 do begin  oplot,cal_curves(kkk,kk,*,0),color =(40*(kk+kkk)+10) mod 254 & endfor & endfor
  
      plot,cal_curves(0,0,*,1),/nodata,yr = [-1,1],/ystyle,title = 'Stokes Q',charsize = cchars, ytitle = 'Q/I',xtitle = 'calibration position'
     for kkk = 0,n_separations-1 do begin & for kk = 0,n_slits-1 do begin  oplot,cal_curves(kkk,kk,*,1),color =(40*(kk+kkk)+10) mod 254 & endfor & endfor

        plot,cal_curves(0,0,*,2),/nodata,yr = [-1,1],/ystyle,title = 'Stokes U',charsize = cchars, ytitle = 'U/I',xtitle = 'calibration position'
   for kkk = 0,n_separations-1 do begin & for kk = 0,n_slits-1 do begin  oplot,cal_curves(kkk,kk,*,2),color =(40*(kk+kkk)+10) mod 254 & endfor & endfor

      plot,cal_curves(0,0,*,3),/nodata,yr = [-1,1],/ystyle,title = 'Stokes V',charsize = cchars, ytitle = 'V/I',xtitle = 'calibration position'
     for kkk = 0,n_separations-1 do begin & for kk = 0,n_slits-1 do begin  oplot,cal_curves(kkk,kk,*,3),color =(40*(kk+kkk)+10) mod 254 & endfor & endfor
  


device,/close
set_plot,'X'

stop

endif

; select only measurements with polarizer in the beam, and no dark slide
dd = reform(pos_icu(*,0))
dd1= reform(pos_icu(*,4))

ff = where(dd eq 1 and dd1 ne 1)
n = n_elements(ff)

temp = fltarr(n,5)
temp(*,0) = pos_icu(ff,0)
temp(*,1) = pos_icu(ff,1)
temp(*,2) = pos_icu(ff,2)
temp(*,3) = pos_icu(ff,3)
temp(*,4) = pos_icu(ff,4)
pos_icu = temp
pos_table = pos_table(ff)
pos_elevation = pos_elevation(ff)
pos_azimuth = pos_azimuth(ff)
light_level = light_level(ff)
light_level_time = light_level_time(ff)


ccal_curves = fltarr(n_separations,n,4)
for kk = 0,n_separations -1 do begin & ccal_curves(kk,*,0) = cal_curves(kk,0,ff,0) & ccal_curves(kk,*,1) = cal_curves(kk,0,ff,1) & ccal_curves(kk,*,2) = cal_curves(kk,0,ff,2) & ccal_curves(kk,*,3) = cal_curves(kk,0,ff,3) & endfor
cal_curves = ccal_curves

; read in additional +-QU measurements, obsolete for SPINOR
if n_elements(add_cal) ne 0 then begin

   print
   print,'Reading in additional calibration files...'
   add_calfiles = bbasefile1(add_cal)
   
for k = 0,n_elements(add_calfiles) -1 do begin & erg = read_directory(input_dir,filter = add_calfiles(k)+'*') & add_calfiles(k) = erg.files(0) & endfor

   add_calvalues = fltarr(n_slits,4,4,2)

 for k = 0,3 do begin
      ffxread,input_dir+add_calfiles(k),dum,hdr
      if n_elements(swap) ne 0 then dum = swap_endian(dum)
      dum = dum -dc

; cut out beams   
      ibeams = fltarr(n_slits,2,dx+1,dy+1,4)

for kk = 0,n_slits-1 do begin &  ii_temp = reform(dum(*,*,0)) & ibeams(kk,0,*,*,0) = ii_temp(posx(kk,0):posx(kk,1),posy(kk,0):posy(kk,1))  & ibeams(kk,1,*,*,0) = ii_temp(posx(kk,0):posx(kk,1),posy(kk,2):posy(kk,3)) & qq_temp = reform(dum(*,*,1)) & ibeams(kk,0,*,*,1) = qq_temp(posx(kk,0):posx(kk,1),posy(kk,0):posy(kk,1)) & ibeams(kk,1,*,*,1) = qq_temp(posx(kk,0):posx(kk,1),posy(kk,2):posy(kk,3)) & uu_temp = reform(dum(*,*,2)) & ibeams(kk,0,*,*,2) = uu_temp(posx(kk,0):posx(kk,1),posy(kk,0):posy(kk,1)) & ibeams(kk,1,*,*,2) = uu_temp(posx(kk,0):posx(kk,1),posy(kk,2):posy(kk,3)) & vv_temp = reform(dum(*,*,3)) & ibeams(kk,0,*,*,3) = vv_temp(posx(kk,0):posx(kk,1),posy(kk,0):posy(kk,1)) & ibeams(kk,1,*,*,3) = vv_temp(posx(kk,0):posx(kk,1),posy(kk,2):posy(kk,3)) & endfor

     ibeams1 = fltarr(n_slits,2,dx_final+1,dy_final+1,4)

  for kk = 0,n_slits-1 do begin & ibeams1(kk,0,*,*,*) =  ibeams(kk,0,posx_final(kk,0):posx_final(kk,1), posy_final(kk,0): posy_final(kk,1),*) & ibeams1(kk,1,*,*,*) =  ibeams(kk,1,posx_final(kk,2):posx_final(kk,3), posy_final(kk,2): posy_final(kk,3),*) & endfor

;if n_elements(ngain) eq 0 then begin
;for kk = 0,n_slits -1 do begin &  for kkk = 0,3 do begin & ibeams1(kk,0,*,*,kkk) =  ibeams1(kk,0,*,*,kkk) /gaintable(kk,0,*,*,kkk) & ibeams1(kk,1,*,*,kkk) =  ibeams1(kk,1,*,*,kkk) / gaintable(kk,1,*,*,kkk)  & endfor & endfor
;endif
 for kk = 0,n_slits -1 do begin &  for kkk = 0,3 do begin & ibeams1(kk,0,*,*,kkk) =  rot(reform(ibeams1(kk,0,*,*,kkk)),aangle(kk),/interp) & ibeams1(kk,1,*,*,kkk) =  rot(reform(ibeams1(kk,1,*,*,kkk)),aangle(kk),/interp)  & endfor & endfor
 ;ibeams_merged =  fltarr(n_slits,dx_final+1,dy_final+1,4)
 ;    for kk = 0,n_slits -1 do begin & for kkk = 0,3 do begin & ibeams_merged(kk,*,*,kkk) =    ( ibeams1(kk,0,*,*,kkk) - ibeams1(kk,1,*,*,kkk) ) / (ibeams1(kk,0,*,*,kkk) + ibeams1(kk,1,*,*,kkk) )& endfor & endfor

   if n_elements(spat_res) eq 0 then begin
      for kk = 0,n_slits-1 do begin  & temp_i = reform(ibeams1(kk,0,*,*,0))  & temp_q = reform(ibeams1(kk,0,*,*,1))  & temp_u = reform(ibeams1(kk,0,*,*,2))  & temp_v = reform(ibeams1(kk,0,*,*,3))  &  ff = where(temp_i gt ilimit*mean(temp_i) and temp_i lt maxi*mean(temp_i))   &  add_calvalues(kk,k,*,0) = [mean( temp_i(ff)),mean( temp_q(ff)),mean( temp_u(ff)),mean( temp_v(ff))] & temp_i = reform(ibeams1(kk,1,*,*,0))  & temp_q = reform(ibeams1(kk,1,*,*,1))  & temp_u = reform(ibeams1(kk,1,*,*,2))  & temp_v = reform(ibeams1(kk,1,*,*,3))  &  ff = where(temp_i gt ilimit*mean(temp_i) and temp_i lt maxi*mean(temp_i)) &  add_calvalues(kk,k,*,1) = [mean( temp_i(ff)),mean( temp_q(ff)),mean( temp_u(ff)),mean( temp_v(ff))] & endfor

; ff = where(ttemp_i gt ilimit*max(ttemp_i)) & 
; & temp_i = median(temp_i,med_width) & temp_q=  median(temp_q,med_width) & temp_u = median(temp_u,med_width) & temp_v = median(temp_v,med_width) 
;& fff = where(finite(temp_i) ne 1) & if fff(0) ne -1 then ttemp_i(fff) = -10.  & fff = where(finite(temp_q) ne 1) & if fff(0) ne -1 then ttemp_i(fff) = -10.  & fff = where(finite(temp_u) ne 1) & if fff(0) ne -1 then ttemp_i(fff) = -10.  & fff = where(finite(temp_v) ne 1) & if fff(0) ne -1 then ttemp_i(fff) = -10.

   endif else begin
    ;  for kk = 0,n_slits-1 do begin 

     ;    for kkk = 0,n_elements(xxr)-2 do begin 
      ;      for kkkk = 0,n_elements(yyr)-2 do begin 
       ;        cal_curves(kk,k,kkk,kkkk,0) = mean(ibeams_merged(kk,xxr(kkk):xxr(kkk+1),yyr(kkkk):yyr(kkkk+1),0)) &  cal_curves(kk,k,kkk,kkkk,1) = mean(ibeams_merged(kk,xxr(kkk):xxr(kkk+1),yyr(kkkk):yyr(kkkk+1),1)/ibeams_merged(kk,xxr(kkk):xxr(kkk+1),yyr(kkkk):yyr(kkkk+1),0)) &  cal_curves(kk,k,kkk,kkkk,2) = mean(ibeams_merged(kk,xxr(kkk):xxr(kkk+1),yyr(kkkk):yyr(kkkk+1),2)/ibeams_merged(kk,xxr(kkk):xxr(kkk+1),yyr(kkkk):yyr(kkkk+1),0)) &  cal_curves(kk,k,kkk,kkkk,3) = mean(ibeams_merged(kk,xxr(kkk):xxr(kkk+1),yyr(kkkk):yyr(kkkk+1),3)/ibeams_merged(kk,xxr(kkk):xxr(kkk+1),yyr(kkkk):yyr(kkkk+1),0))

        ;    endfor
        ; endfor


;   endfor
   endelse
endfor

endif

if n_elements(add_cal) eq 0 then save,cal_curves,pos_icu,pos_table,pos_elevation, pos_azimuth,light_level,light_level_time,dx_final_cal,dy_final_cal,filename = outfile else  save,cal_curves,pos_icu,add_calvalues,pos_table,pos_elevation, pos_azimuth,light_level,light_level_time,dx_final_cal,dy_final_cal,filename = outfile

endif



; check input save file
erg = check_existence(outfile,/read,/change)
outfile = erg(0)

; restore saved polarization vectors
restore,outfile

n = n_elements(cal_curves(0,*,0))

; do plotting
if n_elements(plt) ne 0 then begin
   window,0,title ='Polarization degree in '+num2string(n_slits)+' slits'
   !p.multi = [n_slits,1,n_slits]
for k = 0,n_slits-1 do begin & plot,sqrt( (cal_curves(k,*,1))^2.+ (cal_curves(k,*,2))^2.+ (cal_curves(k,*,3))^2.) & endfor
   if n_elements(nopause) eq 0 then paused
endif



; normalize intensity with straight line over all cal. measurements 
if n_elements(skycal) eq 0 then begin

       for k =0,n_separations - 1 do begin & ii = reform(cal_curves(k,*,0)) & erg = analytger(findgen(n),ii) & cal_curves(k,*,0) = cal_curves(k,*,0) /(erg(1)+erg(0)*findgen(n)) & endfor

       endif else begin
 
           xxx = findgen(n)
           for k =0,n_separations - 1 do begin & ii = reform(cal_curves(k,*,0)) & erg = poly_fit(xxx,ii,4,/double) &  yyy = erg(0)+erg(1)*xxx+erg(2)*xxx^2.+erg(3)*xxx^3.+erg(4)*xxx^4. & cal_curves(k,*,0) = cal_curves(k,*,0) / yyy & endfor
 
  
 


       endelse


print,' Creating input Stokes vectors...'

; create input polarization 
; Calculate Stokes input from calibration unit

; Non-ideal polarizer
; calculation of dichroism
; Note: transmission of incoming U and V is ignored (sq)

px = 1.
py = 0.005
sq = sqrt(px*py)
a = ( px + py )/2.
c = ( px - py )/2.
polmat = setmtxright(a,c,0,0,c,a,0,0,0,0,0,0,0,0,0,0)/( (px + py )/2.)

; Retardance of cal. unit of the observation wavelength
; Non-ideal waveplate

; retardance
retard = retard*!pi/180.

; (Non)-ideal retarder
b = 0.

; calculation of dichroism
px = 1 + b/2.
py = 1 - b/2.

sq = sqrt(px*py)
a = ( px + py )/2.
c = ( px - py )/2.
retmat = setmtxright(a,c,0,0,c,a,0,0,0,0,sq*cos(retard),sq*sin(retard),0,0,-sq*sin(retard),sq*cos(retard))

; Offset angle between POL and RET 
theta_err = theta_err*!pi/180.

stokesin = fltarr(n,4)

; calculate Stokes vector
for t =0,n-1 do begin

        thetaret = pos_icu(t,3)*!pi/180.
        thetapol = pos_icu(t,1)*!pi/180.

; rotation of polarizer
        c = cos(2*thetapol)
        s = sin(2*thetapol)
        rotext1 = setmtxright(1,0,0,0,0,c,s,0,0,-s,c,0,0,0,0,1)
        rotext2 = setmtxright(1,0,0,0,0,c,-s,0,0,s,c,0,0,0,0,1)

; rotation of retarder
        c1 = cos(2*thetaret)
        s1 = sin(2*thetaret)
        rotint1 = setmtxright(1,0,0,0,0,c1,s1,0,0,-s1,c1,0,0,0,0,1)
        rotint2 = setmtxright(1,0,0,0,0,c1,-s1,0,0,s1,c1,0,0,0,0,1)

; rotation between both elements     
        c2 = cos(2*theta_err)
        s2 = sin(2*theta_err)
        rot_err = setmtxright(1,0,0,0,0,c2,s2,0,0,-s2,c2,0,0,0,0,1)

; calculate optical train
if pos_icu(t,2) eq 1 then calibmat = rotint2 ## retmat ## rotint1 ## rot_err ## rotext2 ## polmat ## rotext1 else calibmat = rot_err ## rotext2 ## polmat ## rotext1

; assume unpolarized input if T-matrix is not used
stokesvec = (calibmat ## [1,0,0,0])
stokesvec =  stokesvec/stokesvec(0)

; get T-matrix and polarized input otherwise
if n_elements(tmat_save) ne 0 then begin
   tmtx  = ibis_tmtx(tparams, wavelength, pos_azimuth(t), pos_elevation(t), pos_table(t))
   tmtx = tmtx/tmtx(0,0)
   invec = tmtx##[1,0,0,0]
   stokesvec = (calibmat ## invec)
endif

stokesin(t,*) = stokesvec
endfor

; do plotting
if n_elements(plt) ne 0 then begin

   if n_elements(spat_res) eq 0 then begin
      window,10,title ='Example calibration curves'
      !p.multi = [4,2,2]
      plot,cal_curves( n_separations/2,*,0),/nodata,yr =[.5,1.5]
   for kkk = 0,n_separations-1 do begin &  oplot,cal_curves(kkk,*,0),color =40*kkk+10 & endfor 
      oplot,stokesin(*,0),color =240
      plot,cal_curves( n_separations/2,*,1),/nodata,yr =[-1,1]
   for kkk = 0,n_separations-1 do begin &  oplot,cal_curves(kkk,*,1),color =40*kkk+10 & endfor 
      oplot,stokesin(*,1),color =240

      plot,cal_curves( n_separations/2,*,2),/nodata,yr =[-1,1]
   for kkk = 0,n_separations-1 do begin &  oplot,cal_curves(kkk,*,2),color =40*kkk+10 & endfor 
      oplot,stokesin(*,2),color =240

 plot,cal_curves( n_separations/2,*,3),/nodata,yr =[-1,1]
   for kkk = 0,n_separations-1 do begin &  oplot,cal_curves(kkk,*,3),color =40*kkk+10 & endfor 
      oplot,stokesin(*,3),color =240

   endif

   if n_elements(nopause) eq 0 then paused
 
endif


; get response function X
xxmat = fltarr(n_separations,4,4)
x00 = fltarr(n_separations)

print,' Calculating response function...'

cchi = fltarr(n_separations)

for kkk =0,n_separations -1 do begin 
   
   stokesout = reform(cal_curves(kkk,*,*))
 
; do matrix inversion between input and output
   matrix_inversion,transpose(stokesin),transpose(stokesout),xmat,error

   x00(kkk) = xmat(0,0)
   xmat = xmat / xmat(0,0)
   xxmat(kkk,*,*) = xmat
   xinitial = xmat

   ; get only +-QU measurements for determination of retardance
   polin = reform(pos_icu(*,0))
   retin = reform(pos_icu(*,2))
   pospol = reform(pos_icu(*,1))
   posret = reform(pos_icu(*,3))

   ff = where(retin eq 0 and (abs(pospol) lt 1. or abs(pospol-180) lt 1.) )
   ii = reform(stokesout(ff,0))
   qq = reform(stokesout(ff,1))
   uu = reform(stokesout(ff,2))
   vv = reform(stokesout(ff,3))

   ii = mean(ii)
   qq = mean(qq)
   uu = mean(uu)
   vv = mean(vv)

   dmod = invert(xmat,/double)
   iinv_q = dmod(0,0)*ii + dmod(1,0)*qq + dmod(2,0)*uu + dmod(3,0)*vv
   qinv_q = dmod(0,1)*ii + dmod(1,1)*qq + dmod(2,1)*uu + dmod(3,1)*vv
   uinv_q = dmod(0,2)*ii + dmod(1,2)*qq + dmod(2,2)*uu + dmod(3,2)*vv
   vinv_q = dmod(0,3)*ii + dmod(1,3)*qq + dmod(2,3)*uu + dmod(3,3)*vv

   qinv_q = qinv_q/iinv_q
   uinv_q = uinv_q/iinv_q
   vinv_q = vinv_q/iinv_q
   iinv_q = iinv_q/iinv_q
  
   ff = where(retin eq 0 and (abs(pospol-90) lt 1. or abs(pospol-270) lt 1.) )
   ii = reform(stokesout(ff,0))
   qq = reform(stokesout(ff,1))
   uu = reform(stokesout(ff,2))
   vv = reform(stokesout(ff,3))

   ii = mean(ii)
   qq = mean(qq)
   uu = mean(uu)
   vv = mean(vv)
 
   iinv_minusq = dmod(0,0)*ii + dmod(1,0)*qq + dmod(2,0)*uu + dmod(3,0)*vv
   qinv_minusq = dmod(0,1)*ii + dmod(1,1)*qq + dmod(2,1)*uu + dmod(3,1)*vv
   uinv_minusq = dmod(0,2)*ii + dmod(1,2)*qq + dmod(2,2)*uu + dmod(3,2)*vv
   vinv_minusq = dmod(0,3)*ii + dmod(1,3)*qq + dmod(2,3)*uu + dmod(3,3)*vv

   qinv_minusq = qinv_minusq/iinv_minusq
   uinv_minusq = uinv_minusq/iinv_minusq
   vinv_minusq = vinv_minusq/iinv_minusq
   iinv_minusq = iinv_minusq/iinv_minusq
   
   ff = where(retin eq 0 and (abs(pospol-45) lt 1. or abs(pospol-225) lt 1.))
   ii = reform(stokesout(ff,0))
   qq = reform(stokesout(ff,1))
   uu = reform(stokesout(ff,2))
   vv = reform(stokesout(ff,3))

   ii = mean(ii)
   qq = mean(qq)
   uu = mean(uu)
   vv = mean(vv)

   iinv_u = dmod(0,0)*ii + dmod(1,0)*qq + dmod(2,0)*uu + dmod(3,0)*vv
   qinv_u = dmod(0,1)*ii + dmod(1,1)*qq + dmod(2,1)*uu + dmod(3,1)*vv
   uinv_u = dmod(0,2)*ii + dmod(1,2)*qq + dmod(2,2)*uu + dmod(3,2)*vv
   vinv_u = dmod(0,3)*ii + dmod(1,3)*qq + dmod(2,3)*uu + dmod(3,3)*vv

   qinv_u = qinv_u/iinv_u
   uinv_u = uinv_u/iinv_u
   vinv_u = vinv_u/iinv_u
   iinv_u = iinv_u/iinv_u

   ff = where(retin eq 0 and (abs(pospol-135) lt 1. or abs(pospol-315) lt 1.) )
   ii = reform(stokesout(ff,0))
   qq = reform(stokesout(ff,1))
   uu = reform(stokesout(ff,2))
   vv = reform(stokesout(ff,3))

   ii = mean(ii)
   qq = mean(qq)
   uu = mean(uu)
   vv = mean(vv)
   
   iinv_minusu = dmod(0,0)*ii + dmod(1,0)*qq + dmod(2,0)*uu + dmod(3,0)*vv
   qinv_minusu = dmod(0,1)*ii + dmod(1,1)*qq + dmod(2,1)*uu + dmod(3,1)*vv
   uinv_minusu = dmod(0,2)*ii + dmod(1,2)*qq + dmod(2,2)*uu + dmod(3,2)*vv
   vinv_minusu = dmod(0,3)*ii + dmod(1,3)*qq + dmod(2,3)*uu + dmod(3,3)*vv

   qinv_minusu = qinv_minusu/iinv_minusu
   uinv_minusu = uinv_minusu/iinv_minusu
   vinv_minusu = vinv_minusu/iinv_minusu
   iinv_minusu = iinv_minusu/iinv_minusu

; compare +-QU input/calibrated   
   chi = total(  ([iinv_q,qinv_q,uinv_q,vinv_q] - [1,1,0,0])^2. +  ([iinv_u,qinv_u,uinv_u,vinv_u] - [1,0,1,0])^2. + ([iinv_minusq,qinv_minusq,uinv_minusq,vinv_minusq] - [1,-1,0,0])^2. + ([ iinv_minusu, qinv_minusu, uinv_minusu, vinv_minusu]- [1,0,-1,0])^2. )

   cchi(kkk) = chi

if n_elements(nverbose) eq 0 then begin
   print
   print,' Output - Input add cal'
   print,[iinv_q,qinv_q,uinv_q,vinv_q],[1,1,0,0]
   print,[iinv_u,qinv_u,uinv_u,vinv_u],[1,0,1,0]
   print,[iinv_minusq,qinv_minusq,uinv_minusq,vinv_minusq],[1,-1,0,0]
   print,[ iinv_minusu, qinv_minusu, uinv_minusu, vinv_minusu],[1,0,-1,0]
   print
   print,'Average deviation of add_cal vectors: ',sqrt(chi)/4.
endif

; invert (forward) response
xinv = invert(xinitial,/double)

if n_elements(nverbose) eq 0 then begin
   print
   print,'Xmat: '
   print,xmat
   print
   print
   print,'Inverse'
   print,xinv
   print
   print,'Efficiencies:'
   print,sqrt(total(xmat(1,*)^2.)),sqrt(total(xmat(2,*)^2.)),sqrt(total(xmat(3,*)^2.))
   erg = [sqrt(total(xmat(1,*)^2.)),sqrt(total(xmat(2,*)^2.)),sqrt(total(xmat(3,*)^2.))]


;stop

endif

if n_elements(nverbose) eq 0 then print,' '
if max(erg) gt 0.866 then begin
   print
   print,' !! spinor_cal: Warning !! Efficiencies above 0.866. Seems fishy.' 
   print
endif

; check if X is a Mueller matrix
rr = check_physicality(xmat)
if rr(0) le 0 then print,' Warning : Unphysical matrix !! ',rr(0)
if rr(0) ge 0 then print,' Seems to be ok...'

; do plotting
if n_elements(plt) ne 0 then begin

totout = stokesout

ii1  = xinv(0,0)*stokesout(*,0)+xinv(1,0)*stokesout(*,1)+xinv(2,0)*stokesout(*,2)+xinv(3,0)*stokesout(*,3)
ii2  = xinv(0,1)*stokesout(*,0)+xinv(1,1)*stokesout(*,1)+xinv(2,1)*stokesout(*,2)+xinv(3,1)*stokesout(*,3)
ii3  = xinv(0,2)*stokesout(*,0)+xinv(1,2)*stokesout(*,1)+xinv(2,2)*stokesout(*,2)+xinv(3,2)*stokesout(*,3)
ii4  = xinv(0,3)*stokesout(*,0)+xinv(1,3)*stokesout(*,1)+xinv(2,3)*stokesout(*,2)+xinv(3,3)*stokesout(*,3)

i1 = xinitial(0,0)*stokesin(*,0)+xinitial(1,0)*stokesin(*,1)+xinitial(2,0)*stokesin(*,2)+xinitial(3,0)*stokesin(*,3)
i2 = xinitial(0,1)*stokesin(*,0)+xinitial(1,1)*stokesin(*,1)+xinitial(2,1)*stokesin(*,2)+xinitial(3,1)*stokesin(*,3)
i3 = xinitial(0,2)*stokesin(*,0)+xinitial(1,2)*stokesin(*,1)+xinitial(2,2)*stokesin(*,2)+xinitial(3,2)*stokesin(*,3)
i4 = xinitial(0,3)*stokesin(*,0)+xinitial(1,3)*stokesin(*,1)+xinitial(2,3)*stokesin(*,2)+xinitial(3,3)*stokesin(*,3)

window,0,title = 'Observed cal. measurements and X * input, slit '+num2string(1)
!p.multi = [4,1,4]
plot,stokesout(*,0),/yno
oplot,i1,psym = -1,color = 240
plot,stokesout(*,1),/yno
oplot,i2,psym = -1,color = 240
plot,stokesout(*,2),/yno
oplot,i3,psym = -1,color = 240
plot,stokesout(*,3),/yno
oplot,i4,psym = -1,color = 240

legend1,text =[' Observed calibration curves',' X * Stokes input'],color =[ 256^3.-1,240],ttextcolor = [ 256^3.-1,240],pos = 1,linestyle = fltarr(2),charsize = 1.

window,1,title = 'Created input and X!E-1!N * observed cal.measurements, slit '+num2string(k+1)
!p.multi = [4,1,4]
plot,stokesin(*,0),/yno,yrange = [.9,1.1],/ystyle
oplot,ii1,psym = -1,color = 240
plot,stokesin(*,1),/yno
oplot,ii2,psym = -1,color = 240
plot,stokesin(*,2),/yno
oplot,ii3,psym = -1,color = 240
plot,stokesin(*,3),/yno
oplot,ii4,psym = -1,color = 240

legend1,text =[' Stokes input',' X!E-1!N * observed calibration curves'],color =[256^3.-1,240],ttextcolor = [256^3.-1,240],pos = 1,linestyle = fltarr(2),charsize = 1.

if n_elements(nopause) eq 0 then paused

endif

endfor

if n_elements(nverbose) eq 0 then print,'Saving to: ',outfile1

; save results
save,theta_err,xxmat,retard,cchi,yyr,n_separations,dx_final_cal,dy_final_cal,filename = outfile1

;,add_stokesout

if n_elements(add_cal) ne 0 then begin
   print
   print,total(cchi/4.),retard/!pi*180.
endif


;stop

;paused

;save,stokesout,light_level_time,i1,i2,i3,i4,light_level,filename = '/export/cbeck/work/ATST_EAST/sky_brightness/spinor_cal_210314.sav'
;stop

end

