pro spinor_ff,input_dir=input_dir,output_dir=output_dir,no_av = no_av,beam_ang = beam_ang,plt = plt, verbose = verbose, int_limit = int_limit,flat_file = flat_file,sspeed = sspeed,swap = swap,yyrange = yyrange,full_beam = full_beam,pseudofts = pseudofts,nopause = nopause,check_lines = check_lines,straight = straight,third_order = third_order,nodeskew = nodeskew,addshift = addshift,binning = binning,lamp_gain = lamp_gain,up_lim = up_lim,nolines = nolines, pposx = pposx,help = help

;+
;============================================================================
;
;	procedure : spinor_ff  
;
;	purpose   : calculate gain table for SPINOR data
;
;	author    : cbeck@nso Apr 2013
;
;==============================================================================

if n_elements(help) ne 0 then begin
	print
	print, "usage:   spinor_ff,input_dir=input_dir,output_dir=output_dir,no_av = no_av,beam_ang = beam_ang,plt = plt, verbose = verbose, int_limit = int_limit,flat_file = flat_file,sspeed = sspeed,swap = swap,yyrange = yyrange,full_beam = full_beam,pseudofts = pseudofts,nopause = nopause,check_lines = check_lines,straight = straight,third_order = third_order,nodeskew = nodeskew,addshift = addshift,binning = binning,lamp_gain = lamp_gain,up_lim = up_lim,nolines = nolines,help = help"
	print
	print, "	Calculate gain table for SPINOR data "
        print, "         "
	print
	print, "	Arguments"
        print, "                none"
        print
        print, "         Keywords :"
        print, "              input_dir: source directory with raw data."
        print, "              output_dir: destination directory for reduced data."
        print, "              /no_av: switch to not averaging raw data, but use previously"
        print, "                      created save files."
        print, "              beam_ang: fltarr(1). Rotation of beams, overrules"
        print, "                       value determined in the routine."
        print, "              /plt : switch on plotting (recommended)"
        print, "              /verbose : switch to activate extensive verbal"
        print, "                        output on the terminal."
        print, "              int_limit: fltarr(1). Intensity threshold to"
        print, "                        localize the beams/slits. Fraction of"
        print, "                        maximal intensity."
        print, "              flat_file: number of flat field file to be reduced"
        print, "                        in the source directory."
        print, "               sspeed :  speeds up the convergence in iterate_gain."
        print, "                         > 0 < ~ 10. Too high value leaves line"
        print, "                         residuals."
        print, "               /swap   : do a swap endian of the raw data."
        print, "               yyrange : fltarr(4). Height of the two beams along the slit."
        print, "                         [y0,y1,y2,y3] = lower (y0,y1) and upper"
        print, "                         beam (y3,y4)" 
        print, "               /full_beam: switch between full-sized beam (Ca II IR,"
        print, "                         two hairlines) and part of the beam only (1083 nm, 1565 nm)."
        print, "                         Affects finding beam edges."
        print, "               /pseudofts: keyword for fringe filtering, do not"
        print, "                          use, does not work satifactorily."
        print, "               /nopause: ignore all stop/pause commands."
        print, "               /check_lines: manually select spectral lines"
        print, "               /straight: use straight line fit for line-core"
        print, "                          position along the slit."
        print, "               /third_order: use 3rd order polynomial fit"
        print, "                          for line-core position along the slit."
        print, "               /nodeskew: do not try to deskew flat field spectra."
        print, "               addshift: [dx,dy] add a shift value to align"
        print, "                         the beams in case automatic alignment"
        print, "                         fails persistently. Pixel-accuracy only."
        print, "               binning: fltarr(1). Size of display windows"
        print, "                        will be divided by value to fit on"
        print, "                        small screens."
        print, "               lamp_gain: fltarr(dx,dy), output of  spinor_lamp_ff."
        print, "                         Applies the gain correction from a"
        print, "                         lamp flat to the data."
        print, "               up_lim:   fltarr(1). Upper limit for display"
        print, "                         of intensities. Needed usually when"
        print, "                         the lamp flat is used."
        print, "               /nolines: use if no spectral lines are visible"
        print, "                         in the data. Added for sky polarization "
        print, "                         measurements. Uses a hard-wired position"
        print, "                         in x (line ~195ff)."
        print, "               /help   : switch to active the help text above."
        print
return
endif
;==============================================================================
;-

; default values
if n_elements(int_limit) eq 0 then  int_limit = .1
if n_elements(flat_file) eq 0 then  flat_file = 0
if n_elements(sspeed) eq 0 then  sspeed = .1
if n_elements(beam_ang) eq 0 then  beam_ang = 0.

; rotation of spectrum
aangle = beam_ang

erg = read_directory(input_dir,filter ='*sun.flat* ')
files = erg.files
flatfile = files(flat_file)

; read in and average dark current measurements

if n_elements(no_av) eq 0 then begin
   print,' Averaging dc from file...',flatfile
   n = 8.
   dum = 0.

   erg = check_existence(input_dir+flatfile,/read)
   if erg(1) ne 1 then begin
      print,'!spinor_ff: cannot read ',input_dir+flatfile
      print,' Please check flat field file.'
      print,' Aborting.'
      return
   endif

for k =0,3 do begin & read_spinor,input_dir+flatfile,data,1.+k,hdr &  demod_spinor,data,i,q,u,v & dum = dum + float(i)/n & endfor
for k =0,3 do begin & read_spinor,input_dir+flatfile,data,21.+k,hdr &  demod_spinor,data,i,q,u,v & dum = dum + float(i)/n & endfor

   dc = float(dum)
   erg = check_existence(output_dir+'dc.'+num2string(fix(flat_file))+'.sav',/write,/change)
   dc_file = erg(0)
   save,dc,filename = dc_file
   print,' Saving dc to...',dc_file
   
endif

; average flat field images

if n_elements(no_av) eq 0 then begin
   n = 16.
   dum = 0.

   print,' Averaging ff from file...',flatfile

   erg = check_existence(input_dir+flatfile,/read)
   if erg(1) ne 1 then begin
      print,'!spinor_ff: cannot read ',input_dir+flatfile
      print,' Please check flat field file.'
      print,'Aborting.'
      return
   endif

for k =0,15 do begin & read_spinor,input_dir+flatfile,data,5.+k,hdr &  demod_spinor,data,i,q,u,v & dum = dum + float(i)/n & endfor
   ff_img = float(dum)

   erg = check_existence(output_dir+'ff.'+num2string(fix(flat_file))+'.sav',/write,/change)
   ff_file = erg(0)
   save,ff_img,filename = ff_file
   print,' Saving ff to...',ff_file

endif

; restore averaged dc and ff images
erg = check_existence(output_dir+'dc.'+num2string(fix(flat_file))+'.sav',/read,/change)
dc_file = erg(0)
restore,dc_file
erg = check_existence(output_dir+'ff.'+num2string(fix(flat_file))+'.sav',/read,/change)
ff_file = erg(0)
restore,ff_file

; output file
erg = check_existence(output_dir +'gain.'+num2string(fix(flat_file))+'.sav',/write,/change)
outfile = erg(0)

; subtract dark current
ff_img = ff_img - dc

if n_elements(lamp_gain) ne 0 then begin

ff_img  = ff_img /lamp_gain

endif


; catch intensities below 0
ww = where(ff_img lt 0)
if ww(0) ne -1 then ff_img(ww) = 1.
ww = where(ff_img eq 0)
if ww(0) ne -1 then ff_img(ww) = 1.
ww = where(finite(ff_img) ne 1)
if ww(0) ne -1 then ff_img(ww) = 1.

print,' Trying to determine beam locations...'

ff_img1 = ff_img
tt = size(ff_img)

; switch to distinguish full Stokes and Stokes I only
if tt(0) eq 2 then int_only = 1 else int_only = 0

if int_only eq 0 then ff_img2 = total(ff_img,3)/4. else ff_img2 = ff_img

; determine number of slits (obsolete, SPINOR has single slit)
n_slits = 1
nlobes = [0,tt(1)-1]

; get position values in x and y
posx = fltarr(n_slits,2)
index_slit = 0.
index = 0.
for k =0,n_elements(nlobes)-1 do begin & posx(index_slit,index) = nlobes(k) & index = (index + 1) & if index eq 2 then index_slit = index_slit + 1 & index = index mod 2 & endfor

if n_elements(pposx) ne 0 then begin


   posx(0,0) = pposx(0)
   posx(0,1) = pposx(1)



endif



if n_elements(nolines) ne 0 then begin
   ;posx(0,0) = 185
   ;posx(0,1) = 240

   ;posx(0,0) = 235
   ;posx(0,1) = 325

endif


posy = fltarr(n_slits,4)
posy(*,0) = 0
posy(*,3) = tt(2)-1

rr = total(ff_img,1)
rr = rr/max(rr)
if n_elements(full_beam) eq 0 then begin
   ff = where(rr lt int_limit)
   posy(*,1) = min(ff)
   posy(*,2) = max(ff)
endif else begin
   
   rr = rr(tt(2)/2.-100:tt(2)/2.+100)
   ff = where(rr lt int_limit)
   posy(*,1) = min(ff)+tt(2)/2.-100.
   posy(*,2) = max(ff)+tt(2)/2.-100.
endelse


; overrule: use user-provided values
if n_elements(yyrange) ne 0 then begin
   posy(*,0) = yyrange(0)
   posy(*,1) = yyrange(1)
   posy(*,2) = yyrange(2)
   posy(*,3) = yyrange(3)
endif

; see firs_630_ff.pro for rest of documentation

if n_elements(binning) eq 0 then binning = 1. else binning = float(binning)

if n_elements(up_lim) eq 0 then up_lim = max(ff_img)

if n_elements(plt) ne 0 then begin

   window,0,xs = tt(1),ys = tt(2)/binning,title = 'Beams located'
   tvscl,congrid(ff_img,tt(1),tt(2)/binning,/interp) > 0 < up_lim
for k = 0,n_slits-1 do begin & draw_rec,[posx(k,0),posx(k,1)],[posy(k,0),posy(k,1)]/binning,/device & draw_rec,[posx(k,0),posx(k,1)],[posy(k,2),posy(k,3)]/binning,/device & endfor
   if n_elements(nopause) eq 0 then paused

endif

; get dimension of beams
dx = fltarr(n_slits)
for k = 0,n_slits-1 do begin & dx(k) = posx(k,1)-posx(k,0) & endfor
dx = min(dx)

dy = fltarr(n_slits,2)
for k = 0,n_slits-1 do begin & dy(k,0) = posy(k,1)-posy(k,0) & dy(k,1) = posy(k,3)-posy(k,2) & endfor
dy = min(dy)

for k = 0,n_slits-1 do begin & posx(k,1) =  posx(k,0)+dx & posy(k,1) =  posy(k,0)+dy & posy(k,3) =  posy(k,2)+dy & endfor

if n_elements(plt) ne 0 then begin

window,0,xs = tt(1),ys = tt(2),title = 'beams located'
tvscl,ff_img> 0 < up_lim
for k = 0,n_slits-1 do begin & draw_rec,[posx(k,0),posx(k,1)],[posy(k,0),posy(k,1)],/device & draw_rec,[posx(k,0),posx(k,1)],[posy(k,2),posy(k,3)],/device & endfor

if n_elements(nopause) eq 0 then paused

endif

if int_only eq 0 then ssize = 4 else ssize = 1

; cut out intensity spectra
ii = fltarr(n_slits,2,dx+1,dy+1,ssize)
for k = 0,n_slits-1 do begin & for kk = 0,ssize-1 do begin &  ff_temp = reform(ff_img(*,*,kk)) &  & ii(k,0,*,*,kk) = ff_temp(posx(k,0):posx(k,1),posy(k,0):posy(k,1)) & ii(k,1,*,*,kk) = ff_temp(posx(k,0):posx(k,1),posy(k,2):posy(k,3)) & endfor & endfor




; flip upper beam vertically
for k =0,n_slits-1 do begin & dd = reform(ii(k,1,*,*,0)) & ii(k,1,*,*,0) = reverse(dd,2) & endfor

if n_elements(plt) ne 0 then begin

for k = 0,n_slits -1 do begin & window,k,xs = 2*dx,ys = dy,title='Beams in slit '+num2string(k)+' mod. state 0' & tvscl,ii(k,0,*,*,0)> 0 < up_lim,0,0 & tvscl,ii(k,1,*,*,0)> 0 < up_lim,dx,0 & endfor

if n_elements(nopause) eq 0 then paused

endif

; correlate beams to align them
print,' Aligning upper and lower beam...'

i1 = reform(ii(0,0,*,*,0))
i2 = reform(ii(0,1,*,*,0))
eerg = fltarr(n_slits,2)

if n_elements(nolines) eq 0 then begin

for k =0,n_slits -1 do begin  & erg = shc( reform(ii(k,0,50:dx-50,50:dy-1,0)),reform(ii(k,1,50:dx-50,50:dy-1,0)),/filt) & eerg(k,*) = erg & print,erg & endfor 

endif else begin

for k =0,n_slits -1 do begin  & erg = shc( reform(ii(k,0,*,*,0)),reform(ii(k,1,*,*,0)),/filt) & eerg(k,*) = erg & print,erg & endfor 


endelse


; add any additional shift provided by the user
if n_elements(addshift) ne 0 then  begin
   eerg(*,0) = eerg(*,0)+addshift(0)
   eerg(*,1) = eerg(*,1)+addshift(1)
endif

shiftx = max( abs(eerg(*,0)))
shifty = max( abs(eerg(*,1)))

; define final beam sizes depending on shifts
dx_final = dx - shiftx
dy_final = dy - shifty

ii1 = fltarr(n_slits,2,dx_final+1,dy_final+1,ssize)

posx_final = posy-posy
posy_final = posy-posy

; get final positions to cut out dedending on positive/negative displacement
for k =0,n_slits-1 do begin 
    for kk = 0,ssize -1  do begin
       if eerg(k,0) lt 0 and  eerg(k,1) lt 0  then begin
          ii1(k,0,*,*,kk) = ii(k,0,0:dx_final,0:dy_final,kk)
          ii1(k,1,*,*,kk) = ii(k,1,abs(eerg(k,0)):abs(eerg(k,0))+dx_final,abs(eerg(k,1)):abs(eerg(k,1))+dy_final,kk)

          posx_final(k,0) = 0
          posx_final(k,1) = dx_final
          posx_final(k,2) = abs(eerg(k,0))
          posx_final(k,3) = abs(eerg(k,0))+dx_final
          
          posy_final(k,0) = 0
          posy_final(k,1) = dy_final
          posy_final(k,2) = abs(eerg(k,1))
          posy_final(k,3) = abs(eerg(k,1))+dy_final
                    
       endif 
       if eerg(k,0) ge 0 and  eerg(k,1) ge 0  then begin
          ii1(k,1,*,*,kk) = ii(k,1,0:dx_final,0:dy_final,kk)
          ii1(k,0,*,*,kk) = ii(k,0,abs(eerg(k,0)):abs(eerg(k,0))+dx_final,abs(eerg(k,1)):abs(eerg(k,1))+dy_final,kk)
          
          posx_final(k,2) = 0
          posx_final(k,3) = dx_final
          posx_final(k,0) = abs(eerg(k,0))
          posx_final(k,1) = abs(eerg(k,0))+dx_final
          
          posy_final(k,2) = 0
          posy_final(k,3) = dy_final
          posy_final(k,0) = abs(eerg(k,1))
          posy_final(k,1) = abs(eerg(k,1))+dy_final
          
       endif 
   
       if eerg(k,0) lt 0 and  eerg(k,1) ge 0  then begin
          ii1(k,0,*,*,kk) = ii(k,0,0:dx_final,abs(eerg(k,1)):abs(eerg(k,1))+dy_final,kk)
          ii1(k,1,*,*,kk) = ii(k,1,abs(eerg(k,0)):abs(eerg(k,0))+dx_final,0:dy_final,kk)

          posx_final(k,0) = 0
          posx_final(k,1) = dx_final
          posx_final(k,2) = abs(eerg(k,0))
          posx_final(k,3) = abs(eerg(k,0))+dx_final
          
          posy_final(k,2) = 0
          posy_final(k,3) = dy_final
          posy_final(k,0) = abs(eerg(k,1))
          posy_final(k,1) = abs(eerg(k,1))+dy_final
       endif 

       if eerg(k,0) ge 0 and  eerg(k,1) lt 0  then begin
          ii1(k,1,*,*,kk) = ii(k,1,0:dx_final,abs(eerg(k,1)):abs(eerg(k,1))+dy_final,kk)
          ii1(k,0,*,*,kk) = ii(k,0,abs(eerg(k,0)):abs(eerg(k,0))+dx_final,0:dy_final,kk)

          posx_final(k,2) = 0
          posx_final(k,3) = dx_final
          posx_final(k,0) = abs(eerg(k,0))
          posx_final(k,1) = abs(eerg(k,0))+dx_final
      
          posy_final(k,0) = 0
          posy_final(k,1) = dy_final
          posy_final(k,2) = abs(eerg(k,1))
          posy_final(k,3) = abs(eerg(k,1))+dy_final
          
       endif 
    endfor

 endfor


if n_elements(plt) ne 0 then begin

for k = 0,n_slits -1 do begin & window,k,xs = 2*dx,ys = dy,title='Aligned beams in slit '+num2string(k)+' mod. state 1' & tvscl,ii1(k,0,*,*,0)> 0 < up_lim,0,0 & tvscl,ii1(k,1,*,*,0)> 0 < up_lim,dx,0 & endfor

if n_elements(nopause) eq 0 then paused

endif

dc1 = reform(dc(*,*,0))

lastx = dx_final+1
firsty = 0
lasty = dy_final+1

; calculate gain table

gaintable = fltarr(n_slits,2,dx_final+1,dy_final+1,ssize)
pposref = fltarr(n_slits,2,ssize)
gaintablecoarse = fltarr(n_slits,2,dx_final+1,dy_final+1,ssize)

for k = 0,n_slits -1 do begin 
   for kk =0,1 do begin
      for kkkk = 0,ssize-1 do begin
         iref = reform(ii1(k,kk,*,*,kkkk))
 
         ff = total(iref,2)

  ; determine location of spectral lines
         if kk eq 0 and kkkk eq 0  then begin

            print,' Trying to determine location of spectral lines...'

            iline = ff
            iline = abs(iline/float(max(iline))-1)
            
            n = length(iline)
            ilobes = fltarr(n-4)
            threshh = .2
            ilobeindex = 0.
            for kkk = 5,n-5 do begin
                if iline(kkk) gt max(iline(kkk+1:kkk+4)) and iline(kkk) gt max(iline(kkk-4:kkk-1)) and iline(kkk) gt threshh then begin
                    ilobes(ilobeindex) = kkk
                    ilobeindex = ilobeindex+1
                endif
            endfor
            if ilobeindex ne 2 or n_elements(check_lines) ne 0 then begin
                print,' Automatic detection of spectral lines failed.'
                print,' Please select lines manually:'
                window,0,title =' Please mark 2 spectral lines'
                !p.multi = 0
                plot,iline        
                ilobeindex = 0.
                ilobes = fltarr(n-4)
                while ilobeindex ne 2 do begin
                    wset,0
                    cursor,x,y,/data,/wait
                    if !mouse.button eq 1 then begin
                        ilobes(ilobeindex) = x
                        ilobeindex = ilobeindex+1
                        print,'Line ',ilobeindex
                        print,'at ',x
                        oplot,[x,x],[0,1]
                        wait,.5
                    endif
                    if ilobeindex eq 2 then begin
                        ans = read_keyboard('Selection okay y(es)/n(o) ')
                        if ans eq 'n' then begin
                            ilobeindex = 0
                            plot,iline    
                        endif
                    endif
                endwhile
            endif
            ilobes = ilobes(0:ilobeindex-1)
            ilobes = ilobes(sort(ilobes))
            
            range_telu = [-15,15]+ilobes(0)

            ilo = range_telu(0)
            ihi = range_telu(1)
         ;   wave_lim = [ilobes(0)-30,ilobes(1)+20,sspeed]
            wave_lim = [ilobes(0)-10,ilobes(1)+20,sspeed]
         endif

         print,' Calculating gain table...'

         if n_elements(nolines) eq 0 then begin
         create_gaintables_spinor,ilo,ihi,lastx,firsty,lasty,iref,dc1,gainsave,gaincoarse,posref,wave_lim = wave_lim,verbose =verbose,straight = straight,third_order = third_order,nodeskew = nodeskew
         pposref(k,kk,kkkk) = posref

      endif else begin


         imean = mean(iref(5:*,*))
         
         

         gaintable(k,kk,*,*,kkkk) = iref/imean
         gainsave= iref/imean
         gaincoarse= iref/imean

      endelse



      if n_elements(plt) ne 0 then begin
         window,0,xs = 3*(dx_final+1)/binning,ys = dy_final+1,title = 'Gain table of slit '+num2string(k)+', beam '+num2string(kk)
         tvscl,congrid(iref,(dx_final+1)/binning,dy_final+1,/interp)> 0 < up_lim,0,0
         tvscl,congrid(iref/gainsave,(dx_final+1)/binning,dy_final+1,/interp)> 0 < up_lim,(dx_final+1)/binning,0
         tvscl,congrid(gainsave > .85 < 1.15,(dx_final+1)/binning,dy_final+1,/interp),2*(dx_final+1)/binning,0
         if n_elements(nopause) eq 0 then paused
      endif
      
      gaintable(k,kk,*,*,kkkk) = gainsave
      
      gaintablecoarse(k,kk,*,*,kkkk) = gaincoarse

   endfor

   endfor
endfor


; apply gain correction to input average ff image to get spectral line
; positions in ff data; dunno why I programmed that at all

dum = ff_img

ibeams = fltarr(n_slits,2,dx+1,dy+1,ssize)

if ssize eq 4 then begin
for kk = 0,n_slits-1 do begin &  ii_temp = reform(dum(*,*,0)) & ibeams(kk,0,*,*,0) = ii_temp(posx(kk,0):posx(kk,1),posy(kk,0):posy(kk,1))  & ibeams(kk,1,*,*,0) = ii_temp(posx(kk,0):posx(kk,1),posy(kk,2):posy(kk,3)) & qq_temp = reform(dum(*,*,1)) & ibeams(kk,0,*,*,1) = qq_temp(posx(kk,0):posx(kk,1),posy(kk,0):posy(kk,1)) & ibeams(kk,1,*,*,1) = qq_temp(posx(kk,0):posx(kk,1),posy(kk,2):posy(kk,3)) & uu_temp = reform(dum(*,*,2)) & ibeams(kk,0,*,*,2) = uu_temp(posx(kk,0):posx(kk,1),posy(kk,0):posy(kk,1)) & ibeams(kk,1,*,*,2) = uu_temp(posx(kk,0):posx(kk,1),posy(kk,2):posy(kk,3)) & vv_temp = reform(dum(*,*,3)) & ibeams(kk,0,*,*,3) = vv_temp(posx(kk,0):posx(kk,1),posy(kk,0):posy(kk,1)) & ibeams(kk,1,*,*,3) = vv_temp(posx(kk,0):posx(kk,1),posy(kk,2):posy(kk,3)) & endfor

endif else begin


for kk = 0,n_slits-1 do begin &  ii_temp = reform(dum(*,*,0)) & ibeams(kk,0,*,*,0) = ii_temp(posx(kk,0):posx(kk,1),posy(kk,0):posy(kk,1))  & ibeams(kk,1,*,*,0) = ii_temp(posx(kk,0):posx(kk,1),posy(kk,2):posy(kk,3)) & endfor

endelse

ibeams1 = fltarr(n_slits,2,dx_final+1,dy_final+1,ssize)

for kk = 0,n_slits-1 do begin & ibeams1(kk,0,*,*,*) =  ibeams(kk,0,posx_final(kk,0):posx_final(kk,1), posy_final(kk,0): posy_final(kk,1),*) & ibeams1(kk,1,*,*,*) =  ibeams(kk,1,posx_final(kk,2):posx_final(kk,3), posy_final(kk,2): posy_final(kk,3),*) & endfor

for kk = 0,n_slits -1 do begin &  for kkk = 0,ssize-1  do begin & ibeams1(kk,0,*,*,kkk) =  ibeams1(kk,0,*,*,kkk) /gaintable(kk,0,*,*,kkk) & ibeams1(kk,1,*,*,kkk) =  ibeams1(kk,1,*,*,kkk) / gaintable(kk,1,*,*,kkk)  & endfor & endfor

for kk = 0,n_slits-1 do begin & for kkk = 0,ssize-1  do begin & ff = reform(total(ibeams1(kk,0,*,*,kkk),4)) & xx = findgen(dx_final+1) & erg = poly_fit(xx,ff(xx),3,/double) & yy = erg(0)+erg(1)*findgen(dx_final+1)+erg(2)*findgen(dx_final+1)^2.+erg(3)*findgen(dx_final+1)^3. & yy = yy / mean(yy) & for kkkk = 0,dy_final do begin & ibeams1(kk,0,*,kkkk,kkk) = ibeams1(kk,0,*,kkkk,kkk) / yy & endfor  & ff = reform(total(ibeams1(kk,1,*,*,kkk),4)) & xx = findgen(dx_final+1) & erg = poly_fit(xx,ff(xx),3,/double) & yy = erg(0)+erg(1)*findgen(dx_final+1)+erg(2)*findgen(dx_final+1)^2.+erg(3)*findgen(dx_final+1)^3. & yy = yy / mean(yy) & for kkkk = 0,dy_final do begin & ibeams1(kk,1,*,kkkk,kkk) = ibeams1(kk,1,*,kkkk,kkk) / yy & endfor & endfor & endfor

llinepos = fltarr(n_slits,2,dy_final+1,ssize)

if n_elements(nolines) eq 0 then begin

for kk = 0,n_slits-1 do begin & for kkk = 0,ssize-1  do begin & ff = reform(ibeams1(kk,0,*,*,kkk)) & for kkkk = 0,dy_final do begin & iline = reform(ff(*,kkkk)) & erg = min(iline(pposref(kk,0,kkk)-15:pposref(kk,0,kkk)+15),posmin) & posmin = posmin + pposref(kk,0,kkk)-15 & lpff,iline(posmin-5:posmin+5),pposmin & pposmin = pposmin + posmin -5 & llinepos(kk,0,kkkk,kkk) = pposmin & endfor &  ff = reform(ibeams1(kk,1,*,*,kkk)) & for kkkk = 0,dy_final do begin & iline = reform(ff(*,kkkk)) & erg = min(iline(pposref(kk,0,kkk)-15:pposref(kk,0,kkk)+15),posmin) & posmin = posmin + pposref(kk,0,kkk)-15 & lpff,iline(posmin-5:posmin+5),pposmin & pposmin = pposmin + posmin -5 & llinepos(kk,1,kkkk,kkk) = pposmin & endfor & endfor & endfor 

endif


; possible fringes correction; ignore, does not work correctly
if n_elements(pseudofts) ne 0 then begin

ftsread,data,8500,8600,xlam = xlam  ; 29.08.09

data = swap_endian(data)
data = data/10000.
xlam = xlam/10.


dum = ff_img
ibeams = fltarr(n_slits,2,dx+1,dy+1,ssize)
for kk = 0,n_slits-1 do begin &  ii_temp = reform(dum(*,*,0)) & ibeams(kk,0,*,*,0) = ii_temp(posx(kk,0):posx(kk,1),posy(kk,0):posy(kk,1))  & ibeams(kk,1,*,*,0) = ii_temp(posx(kk,0):posx(kk,1),posy(kk,2):posy(kk,3)) & endfor

for k =0,n_slits-1 do begin & dd = reform(ibeams(k,1,*,*,0)) & ibeams(k,1,*,*,0) = reverse(dd,2) & endfor

ibeams1 = fltarr(n_slits,2,dx_final+1,dy_final+1,ssize)
for kk = 0,n_slits-1 do begin & ibeams1(kk,0,*,*,*) =  ibeams(kk,0,posx_final(kk,0):posx_final(kk,1), posy_final(kk,0): posy_final(kk,1),*) & ibeams1(kk,1,*,*,*) =  ibeams(kk,1,posx_final(kk,2):posx_final(kk,3), posy_final(kk,2): posy_final(kk,3),*) & endfor

for kk = 0,n_slits -1 do begin &  for kkk = 0,ssize-1  do begin & ibeams1(kk,0,*,*,kkk) =  ibeams1(kk,0,*,*,kkk) /gaintable(kk,0,*,*,kkk) & ibeams1(kk,1,*,*,kkk) =  ibeams1(kk,1,*,*,kkk) / gaintable(kk,1,*,*,kkk)  & endfor & endfor

i1 = reform(ibeams1(0,0,*,*,0))
i2 = reform(ibeams1(0,1,*,*,0))

rr = size(i1)
tt1 = size(data)

pseudo_fts = fltarr(tt1(1),rr(2))
for k = 0,rr(2)-1 do begin & pseudo_fts(*,k) = data & endfor

pseudofts1 = congrid(pseudo_fts(12200:27300,*),rr(1),rr(2))   ; 12.5.2013

;plot,total(i1,2)/(1.5*10^8.) 
;oplot,total(pseudofts1,2)/448.,color =240 
;blink,i1,pseudofts1

iline1 = total(i1,2)/float(rr(2))
iline2 = total(i2,2)/float(rr(2))
iline1_old = iline1

fft_filter = 30 ; apr 2011

iline1 = iline1_old

; align with beam 1
erg = shc(i1,pseudofts1,/filt,/interp)
pseudofts1 = shift_frac(pseudofts1,erg(0),0,'C')
fts_line1 = total(pseudofts1,2)/float(rr(2))*10000.
iiline1 = iline1
norm1 = iiline1(0)
iiline1 = iiline1/iiline1(0)
fts_line1  = fts_line1/fts_line1(0)
corr = fts_line1/iiline1
ttt = size(iiline1)
erg = poly_fit(findgen(ttt(1)),corr,3,/double)
yy = erg(0)+erg(1)*findgen(ttt(1))+erg(2)*findgen(ttt(1))^2.+erg(3)*findgen(ttt(1))^3.
fts_line1 = fts_line1/yy

;alpha = .5
;sigma = 1.
;kernel =  1./(sqrt(2*!pi)*sigma) * exp( - (findgen(rr(1))-rr(1)/2.)^2./(2.*sigma^2.))
;fts_line2 = convol( (fts_line1+alpha) / (1. + alpha),kernel,/center,/edge_truncate)
;fts_line2 = fts_line2/max(fts_line2)*max(fts_line1)

ccor1 = i1 - i1
for k = 0,rr(2)-1 do begin & ccor1(*,k) = i1(*,k)/fts_line1 & endfor
for k = 0,rr(2)-1 do begin & erg = fft(reform(ccor1(*,k)),-1) & gg = erg-erg & gg(0:fft_filter,*) = erg(0:fft_filter,*) & ccor1(*,k) = fft(gg,1) & endfor

ccor1= ccor1 /mean(ccor1)

; align with beam 1
erg = shc(i2,pseudofts1,/filt,/interp)
pseudofts1 = shift_frac(pseudofts1,erg(0),0,'C')
fts_line2 = total(pseudofts1,2)/float(rr(2))*10000.
iiline2 = iline2
norm2 = iiline2(0)
iiline2 = iiline2/iiline2(0)
fts_line1  = fts_line1/fts_line1(0)
corr = fts_line1/iiline2
ttt = size(iiline2)
erg = poly_fit(findgen(ttt(1)),corr,3,/double)
yy = erg(0)+erg(1)*findgen(ttt(1))+erg(2)*findgen(ttt(1))^2.+erg(3)*findgen(ttt(1))^3.
fts_line1 = fts_line1/yy

ccor2 = i1 - i1
for k = 0,rr(2)-1 do begin & ccor2(*,k) = i2(*,k)/fts_line1 & endfor
for k = 0,rr(2)-1 do begin & erg = fft(reform(ccor2(*,k)),-1) & gg = erg-erg & gg(0:fft_filter,*) = erg(0:fft_filter,*) & ccor2(*,k) = fft(gg,1) & endfor

ccor2= ccor2 /mean(ccor2)

gaintable(0,0,*,*) = gaintable(0,0,*,*) * ccor1
gaintable(0,1,*,*) = gaintable(0,1,*,*) * ccor2

endif



print,' Saving gain to...',outfile

save,aangle,dx,dy,dx_final,dy_final,posx,posy,posx_final,posy_final,gaintable,gaintablecoarse,n_slits,pposref,llinepos,filename = outfile

if n_elements(plt) ne 0 then wdelete,0

;ende:
;stop

end

